/*
 * $URL$
 * 
 * Copyright (C) 2022 by Arizona State University and Mark Robinson.
 * All rights reserved.
 *
 * $Author$
 *   $Date$
 *    $Rev$
 *
 * (description of this file here)
 *
 */
#include "decompand.h"
#include <stdlib.h>
#include <stdio.h>
#include "special_pixels.h"

uint8_t compand(struct shadowcam_image *image, uint16_t dn) {
  for (int i=0; i<6; i++)
    if (dn < image->xterm[i])
      return ((dn >> i) + image->bterm[i]) & 0xff;

  printf("Failed to decompand value: %d\n", dn);
  exit(1);
  return(0);
}

void decompand(struct shadowcam_image *image) {
  puts("Decompanding.");

  uint8_t companding_table[4096];

  for (uint16_t dn12 = 0; dn12 < 4096; dn12++) {
    companding_table[dn12] = compand(image, dn12);
  }

  uint16_t decompanding_table[256];

  for (uint16_t dn=0; dn < 256; dn++) {
    uint16_t min = 9999, max = 0;

    // find the min/max 12-bit dn of the first range that compands to this 8-bit dn.
    for (uint16_t dn12=0; dn12 < 4096; dn12++) {
      if (companding_table[dn12] == dn) {
        if (min == 9999) {
          min = dn12;
        }

        max = dn12;
      }

      if (min != 9999 && companding_table[dn12] != dn)
        break;
    }

    decompanding_table[dn] = (min + max) / 2;
  }

  uint32_t size = SHC_CHANNELS * SHC_AFE_WIDTH * image->lines;

  for(uint32_t i = 0; i < size; i++)
    if (!is_special(image->data[i]))
      image->data[i] = decompanding_table[(uint16_t) image->data[i]];
}

