/*
 * $URL$
 * 
 * Copyright (C) 2022 by Arizona State University and Mark Robinson.
 * All rights reserved.
 *
 * $Author$
 *   $Date$
 *    $Rev$
 *
 * (description of this file here)
 *
 */
#include "flat_field.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include "config_path.h"
#include "special_pixels.h"

void read_flat_csv(const char* filename, struct shadowcam_image* image, double *flat_coeff) {
  FILE *file = fopen(filename, "rb");
  if (file == NULL) {
    printf("fopen of %s failed with errno %d\n", filename, errno);
    exit(1);
  }

  char buffer[1024];

  for (uint16_t column = 0; column < SHC_SCENE; column++) {
    // find a non-comment/header line
    while(fgets(buffer, sizeof(buffer), file))
      if (strncmp(buffer, "#", 1) != 0)
        break;

    double coeff_tmp[SHC_CHANNELS * 2];
    char *token = strtok(buffer, ",");
    if (token == NULL) {
      printf("Expected ',' in %s, loading CSV failed.\n", filename);
      exit(1);
    }

    for (int i=0; i < SHC_CHANNELS * 2; i++) {
      token = strtok(NULL, ",");
      if (token == NULL) {
        printf("Expected ',' in %s, loading CSV failed.\n", filename);
        exit(1);
      }
      coeff_tmp[i] = strtod(token, NULL);
    }

    for (uint8_t channel = 0; channel < SHC_CHANNELS; channel++)
      flat_coeff[channel * SHC_SCENE + column] = coeff_tmp[channel + SHC_CHANNELS * image->tdi_factor];
  }
}

void flat_field(struct shadowcam_image *image) {
  puts("Flat fielding.");

  double *flat_coeff = malloc(sizeof(double) * SHC_CHANNELS * SHC_SCENE);
  if (flat_coeff == NULL) {
    printf("malloc failed at %s:%d\n", __FILE__, __LINE__);
    exit(1);
  }

  puts("  Reading flat field CSV.");

  char *csv_path = config_file_path("flat_field.csv");
  read_flat_csv(csv_path, image, flat_coeff);
  free(csv_path);

  puts("  Applying flat field correction.");

  for (uint32_t line=0; line < image->lines; line++)
    for (uint8_t channel=0; channel < SHC_CHANNELS; channel++)
      for (uint16_t column=0; column < SHC_SCENE; column++) {
        uint16_t sample = column + SHC_SCENE_OFFSET;

        if (!is_special(image->data[data_index(line, channel, sample)]))
          image->data[data_index(line, channel, sample)] = image->data[data_index(line, channel, sample)] / flat_coeff[channel * SHC_SCENE + column];
      }

  free(flat_coeff);
}
