/*
 * $URL$
 * 
 * Copyright (C) 2022 by Arizona State University and Mark Robinson.
 * All rights reserved.
 *
 * $Author$
 *   $Date$
 *    $Rev$
 *
 * (description of this file here)
 *
 */
#ifndef SHADOWCAM_IMAGE_H
#define SHADOWCAM_IMAGE_H

#include <stdint.h>
#include "constants.h"
#include <stdio.h>

struct shadowcam_image {
  char *filename;
  FILE *file;
  uint32_t data_offset;
  char tdi_direction;
  uint8_t tdi_factor;
  uint16_t gain[SHC_CHANNELS];
  int16_t offset[SHC_CHANNELS];
  uint16_t bterm[6];
  uint16_t xterm[6];
  double fpa_a_temp;
  double line_rate_ms;
  uint32_t lines;
  float *data;
  uint16_t samples;
  char *target_name;
  uint16_t line_rate_code;
  uint16_t preroll;
  uint16_t klines;
  char *execution_time;
  char *execution_spacecraft_time;
  double start_time_offset;
  char *preroll_time;
  char *start_time;
  char *stop_time;
  double fpa_b_temp;
  double fpga_temp;
  double telescope_temp;
  char *mode_test;
  char *pds_identifier;
  char *mission_phase;
  uint8_t dqi;
  int32_t naif_frame_code;
  uint8_t sample_first_pixel;
  uint16_t bias_spread;
  uint16_t bias_spread_ch0;
  uint16_t bias_spread_ch1;
  uint16_t bias_spread_ch2;
  uint16_t bias_spread_ch3;
  uint16_t bias_spread_ch4;
  uint16_t bias_spread_ch5;
  char *dqi_fpa_out_of_bounds;
  char *dqi_under_exposed;
  char *dqi_missing_data;
  char *dqi_missing_spice;
  char *dqi_uncalibratable;
  char *dqi_corruption_detected;
};

struct shadowcam_image* init_shadowcam_image(const char* filename);
void free_shadowcam_image(struct shadowcam_image* image);
void ensure_shadowcam_image_populated(struct shadowcam_image *image);

size_t data_index(uint32_t line, uint8_t channel, uint16_t pixel);

#endif
